(function() {
  var Authentication, BackendMessage, Connection, EventEmitter, FrontendMessage, Query, fs, net, util;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  util = require('util');
  net = require('net');
  fs = require('fs');
  EventEmitter = require('events').EventEmitter;
  FrontendMessage = require('./frontend_message');
  BackendMessage = require('./backend_message');
  Authentication = require('./authentication');
  Query = require('./query');
  Connection = (function() {
    __extends(Connection, EventEmitter);
    function Connection(connectionOptions) {
      var _base, _base2;
      this.connectionOptions = connectionOptions;
      (_base = this.connectionOptions).host || (_base.host = 'localhost');
      (_base2 = this.connectionOptions).port || (_base2.port = 5433);
      this.connected = false;
      this.busy = true;
      this.queue = [];
      this.parameters = {};
      this.key = null;
      this.pid = null;
      this.transactionStatus = null;
      this.incomingData = new Buffer(0);
    }
    Connection.prototype.connect = function(callback) {
      this.connectedCallback = callback;
      this.connection = net.createConnection(this.connectionOptions.port, this.connectionOptions.host);
      return this.connection.on('connect', __bind(function() {
        this.connected = true;
        this._bindEventListeners();
        if (this.connectionOptions.ssl) {
          this._writeMessage(new FrontendMessage.SSLRequest);
          return this.connection.once('data', __bind(function(buffer) {
            var conn, sslOptions, starttls, tls, _ref;
            if ('S' === buffer.toString('utf-8')) {
              tls = require('tls');
              starttls = require('./starttls');
              sslOptions = {
                key: this.connectionOptions.key,
                cert: this.connectionOptions.cert,
                ca: this.connectionOptions.ca
              };
                            if ((_ref = sslOptions.key) != null) {
                _ref;
              } else {
                sslOptions.key = fs.readFileSync("" + __dirname + "/../res/default-client-key.pem");
              };
              return conn = starttls(this.connection, sslOptions, __bind(function() {
                if (!conn.authorized && this.connectionOptions.rejectUnauthorized) {
                  conn.end();
                  this.disconnect();
                  return this.emit('error', new Error(conn.authorizationError));
                } else {
                  if (!conn.authorized) {
                    this.emit('warn', conn.authorizationError);
                  }
                  this.connection = conn;
                  this._bindEventListeners();
                  return this._handshake();
                }
              }, this));
            } else if (this.connectionOptions.ssl === true || this.connectionOptions.ssl === 'required') {
              return this.emit('error', new Error("The server does not support SSL connection"));
            } else {
              return this._handshake();
            }
          }, this));
        } else {
          return this._handshake();
        }
      }, this));
    };
    Connection.prototype._bindEventListeners = function() {
      this.connection.on('close', this._onClose.bind(this));
      this.connection.on('error', this._onError.bind(this));
      return this.connection.on('timeout', this._onTimeout.bind(this));
    };
    Connection.prototype.disconnect = function() {
      return this.connection.end();
    };
    Connection.prototype._schedule = function(job) {
      if (this.busy) {
        this.queue.push(job);
        this.emit('queue', job);
      } else {
        job.execute();
      }
      return job;
    };
    Connection.prototype.query = function(sql, callback) {
      return this._schedule(new Query(this, sql, callback));
    };
    Connection.prototype._handshake = function() {
      var authenticationHandler;
      this.connection.on('data', this._onData.bind(this));
      this._writeMessage(new FrontendMessage.Startup(this.connectionOptions.user, this.connectionOptions.database));
      authenticationHandler = __bind(function(msg) {
        switch (msg.method) {
          case Authentication.methods.OK:
            return this.once('ReadyForQuery', __bind(function(msg) {
              this.busy = false;
              return this.connectedCallback(this);
            }, this));
          case Authentication.methods.CLEARTEXT_PASSWORD:
          case Authentication.methods.MD5_PASSWORD:
            this._writeMessage(new FrontendMessage.Password(this.connectionOptions.password, msg.method, {
              salt: msg.salt,
              user: this.connectionOptions.user
            }));
            return this.once('Authentication', authenticationHandler);
          default:
            throw new Error("Autentication method " + msg.method + " not supported.");
        }
      }, this);
      this.once('Authentication', authenticationHandler);
      this.on('ParameterStatus', __bind(function(msg) {
        return this.parameters[msg.name] = msg.value;
      }, this));
      this.on('BackendKeyData', __bind(function(msg) {
        var _ref;
        return _ref = [msg.pid, msg.key], this.pid = _ref[0], this.key = _ref[1], _ref;
      }, this));
      return this.on('ReadyForQuery', __bind(function(msg) {
        var job;
        this.transactionStatus = msg.transactionStatus;
        if (this.queue.length > 0) {
          job = this.queue.shift();
          this.emit('dequeue', job);
          return job.execute();
        } else {
          return this.emit('ready', this);
        }
      }, this));
    };
    Connection.prototype._onData = function(buffer) {
      var bufferedData, message, size, _results;
      if (this.incomingData.length === 0) {
        this.incomingData = buffer;
      } else {
        bufferedData = new Buffer(this.incomingData.length + buffer.length);
        this.incomingData.copy(bufferedData);
        buffer.copy(bufferedData, this.incomingData.length);
        this.incomingData = bufferedData;
      }
      size = this.incomingData.readUInt32(1);
      _results = [];
      while (size + 1 <= this.incomingData.length) {
        console.log(this.incomingData.slice(0, size + 1));
        message = BackendMessage.fromBuffer(this.incomingData.slice(0, size + 1));
        this.emit('message', message);
        this.emit(message.event, message);
        this.incomingData = this.incomingData.slice(size + 1);
        _results.push(size = this.incomingData.readUInt32(1));
      }
      return _results;
    };
    Connection.prototype._onClose = function() {
      this.connected = false;
      return this.emit('close');
    };
    Connection.prototype._onTimeout = function() {
      return this.emit('timeout');
    };
    Connection.prototype._onError = function(exception) {
      return this.emit('error', exception);
    };
    Connection.prototype._writeMessage = function(msg, callback) {
      return this.connection.write(msg.toBuffer(), null, callback);
    };
    return Connection;
  })();
  module.exports = Connection;
}).call(this);
